/*
	(c) 2021 Dmitry Grinberg   https://dmitry.gr
	Non-commercial use only OR licensing@dmitry.gr
*/


#pragma GCC optimize ("Os")


#include <string.h>
#include <stdarg.h>
#include "printf.h"


static void StrPrvPrintfEx_number(uint64_t number, bool zeroExtend, uint32_t padToLength)
{
	char buf[64];
	uint32_t idx = sizeof(buf) - 1;
	uint32_t numPrinted = 0;
	bool neg = false;
	uint32_t chr, i;
	
	
	if (padToLength > 63)
		padToLength = 63;
	
	buf[idx--] = 0;	//terminate
	
	
	do {
		
		chr = number & 0x0f;
		number = number >> 4;
		buf[idx--] = (chr >= 10)?(chr + 'A' - 10):(chr + '0');
		
		numPrinted++;
		
	} while (number);
	
	if (neg) {
	
		buf[idx--] = '-';
		numPrinted++;
	}
	
	if (padToLength > numPrinted)
		padToLength -= numPrinted;
	else
		padToLength = 0;
	
	while (padToLength--) {
		
		buf[idx--] = zeroExtend ? '0' : ' ';
		numPrinted++;
	}
	
	idx++;
	for (i = 0; i < numPrinted; i++)
		prPutchar((buf + idx)[i]);
}

void prvRaw(const char* fmtStr, va_list vl)
{	
	char c, t;
	
	while((c = *fmtStr++) != 0){
		
		if (c == '%') {
			
			uint32_t padToLength = 0, i;
			bool zeroExtend = false;
			const char* str;
			
more_fmt:
			switch (c = *fmtStr++) {
				
				case '%':
				default:	
					prPutchar(c);
					break;
				
				case 'c':
					
					t = va_arg(vl,unsigned int);
					prPutchar(t);
					break;
				
				case 's':
					str = va_arg(vl,char*);
					if (!str)
						str = "(null)";
					while ((c = *str++))
						prPutchar(c);
					break;
				
				case '0':
					
					if (!zeroExtend && !padToLength) {
						
						zeroExtend = true;
						goto more_fmt;
					}
				
				case '1':
				case '2':
				case '3':
				case '4':
				case '5':
				case '6':
				case '7':
				case '8':
				case '9':
					padToLength = (padToLength * 10) + c - '0';
					goto more_fmt;
				
				case 'x':
				case 'X':
					StrPrvPrintfEx_number(va_arg(vl,uint32_t), zeroExtend, padToLength);
					break;
					
				case 'l':
					goto more_fmt;

			}
		}
		else
			prPutchar(c);
	}
}

void prRaw(const char *format, ...)
{
	va_list vl;
	
	va_start(vl, format);
	prv(format, vl);
	va_end(vl);
}
